/*
 * Fabric3
 * Copyright (c) 2009-2013 Metaform Systems
 *
 * Fabric3 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version, with the
 * following exception:
 *
 * Linking this software statically or dynamically with other
 * modules is making a combined work based on this software.
 * Thus, the terms and conditions of the GNU General Public
 * License cover the whole combination.
 *
 * As a special exception, the copyright holders of this software
 * give you permission to link this software with independent
 * modules to produce an executable, regardless of the license
 * terms of these independent modules, and to copy and distribute
 * the resulting executable under terms of your choice, provided
 * that you also meet, for each linked independent module, the
 * terms and conditions of the license of that module. An
 * independent module is a module which is not derived from or
 * based on this software. If you modify this software, you may
 * extend this exception to your version of the software, but
 * you are not obligated to do so. If you do not wish to do so,
 * delete this exception statement from your version.
 *
 * Fabric3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the
 * GNU General Public License along with Fabric3.
 * If not, see <http://www.gnu.org/licenses/>.
*/
package org.fabric3.fabric.contract;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.oasisopen.sca.annotation.Reference;

import org.fabric3.model.type.contract.ServiceContract;
import org.fabric3.spi.contract.ContractMatcher;
import org.fabric3.spi.contract.ContractMatcherExtension;
import org.fabric3.spi.contract.MatchResult;

/**
 * The default ContractMatcher implementation.
 */
public class DefaultContractMatcher implements ContractMatcher {
    private Map<Key, ContractMatcherExtension<?, ?>> cache = new HashMap<Key, ContractMatcherExtension<?, ?>>();

    @Reference
    public void setMatcherExtensions(List<ContractMatcherExtension<?, ?>> matchers) {
        cache.clear();
        for (ContractMatcherExtension<?, ?> matcher : matchers) {
            addMatcherExtension(matcher);
        }
    }

    public void addMatcherExtension(ContractMatcherExtension<?, ?> matcher) {
        Key key = new Key(matcher.getSource(), matcher.getTarget());
        cache.put(key, matcher);
    }

    @SuppressWarnings({"unchecked"})
    public MatchResult isAssignableFrom(ServiceContract source, ServiceContract target, boolean reportErrors) {
        Key key = new Key(source.getClass(), target.getClass());
        ContractMatcherExtension matcher = cache.get(key);
        if (matcher == null) {
            // this is a programming error
            String name = ContractMatcherExtension.class.getSimpleName();
            throw new AssertionError(name + " not found for converting from " + source.getClass() + " to " + target.getClass());
        }
        return matcher.isAssignableFrom(source, target, reportErrors);
    }

    private class Key {
        private Class<? extends ServiceContract> source;
        private Class<? extends ServiceContract> target;

        private Key(Class<? extends ServiceContract> source, Class<? extends ServiceContract> target) {
            this.source = source;
            this.target = target;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Key that = (Key) o;

            return !(source != null ? !source.equals(that.source) : that.source != null)
                    && !(target != null ? !target.equals(that.target) : that.target != null);

        }

        @Override
        public int hashCode() {
            int result = source != null ? source.hashCode() : 0;
            result = 31 * result + (target != null ? target.hashCode() : 0);
            return result;
        }
    }
}
